"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = __importDefault(require("."));
const utils_1 = require("../utils");
class HomeKitDeviceBulb extends _1.default {
    config;
    tplinkDevice;
    desiredLightState = {};
    constructor(platform, config, homebridgeAccessory, tplinkDevice) {
        super(platform, config, homebridgeAccessory, tplinkDevice, 5 /* Categories.LIGHTBULB */);
        this.config = config;
        this.tplinkDevice = tplinkDevice;
        const primaryService = this.addLightbulbService({
            supportsBrightness: tplinkDevice.supportsBrightness,
            supportsColorTemperature: tplinkDevice.supportsColorTemperature,
            supportsColor: tplinkDevice.supportsColor,
        });
        if (platform.config.addCustomCharacteristics &&
            tplinkDevice.supportsEmeter) {
            this.addEnergyCharacteristics(primaryService);
        }
        else {
            this.removeEnergyCharacteristics(primaryService);
        }
        this.getLightState = (0, utils_1.deferAndCombine)((requestCount) => {
            this.log.debug(`executing deferred getLightState count: ${requestCount}`);
            return this.tplinkDevice.getSysInfo().then((si) => {
                return si.light_state;
            });
        }, platform.config.waitTimeUpdate);
        this.setLightState = (0, utils_1.deferAndCombine)((requestCount) => {
            this.log.debug(`executing deferred setLightState count: ${requestCount}`);
            if (Object.keys(this.desiredLightState).length === 0) {
                this.log.warn('setLightState called with empty desiredLightState');
                return Promise.resolve(true);
            }
            const ret = this.tplinkDevice.lighting.setLightState(this.desiredLightState);
            this.desiredLightState = {};
            return ret;
        }, platform.config.waitTimeUpdate, (value) => {
            this.desiredLightState = Object.assign(this.desiredLightState, value);
        });
        this.getRealtime = (0, utils_1.deferAndCombine)(() => {
            return this.tplinkDevice.emeter.getRealtime();
        }, platform.config.waitTimeUpdate);
    }
    /**
     * Aggregates getLightState requests
     *
     * @private
     */
    getLightState;
    /**
     * Aggregates setLightState requests
     *
     * @private
     */
    setLightState;
    /**
     * Aggregates getRealtime requests
     *
     * @private
     */
    getRealtime;
    addLightbulbService({ supportsBrightness, supportsColorTemperature, supportsColor, }) {
        const { Lightbulb } = this.platform.Service;
        const { Characteristic } = this.platform;
        const lightbulbService = this.homebridgeAccessory.getService(Lightbulb) ??
            this.addService(Lightbulb, this.name);
        const onCharacteristic = (0, utils_1.getOrAddCharacteristic)(lightbulbService, Characteristic.On);
        onCharacteristic
            .onGet(() => {
            this.getLightState().catch(this.logRejection.bind(this)); // this will eventually trigger update
            return this.tplinkDevice.sysInfo.light_state.on_off === 1; // immediately returned cached value
        })
            .onSet(async (value) => {
            this.log.info(`Setting On to: ${value}`);
            if (typeof value === 'boolean') {
                await this.setLightState({ on_off: value ? 1 : 0 });
                return;
            }
            this.log.warn('setValue: Invalid On:', value);
        });
        this.tplinkDevice.on('lightstate-on', () => {
            this.updateValue(lightbulbService, onCharacteristic, true);
        });
        this.tplinkDevice.on('lightstate-sysinfo-on', () => {
            this.updateValue(lightbulbService, onCharacteristic, true);
        });
        this.tplinkDevice.on('lightstate-off', () => {
            this.updateValue(lightbulbService, onCharacteristic, false);
        });
        this.tplinkDevice.on('lightstate-sysinfo-off', () => {
            this.updateValue(lightbulbService, onCharacteristic, false);
        });
        const onUpdateListener = (lightState) => {
            if (lightState.on_off != null) {
                this.updateValue(lightbulbService, onCharacteristic, lightState.on_off === 1);
            }
        };
        this.tplinkDevice.on('lightstate-update', onUpdateListener);
        this.tplinkDevice.on('lightstate-sysinfo-update', onUpdateListener);
        if (supportsBrightness) {
            this.addBrightnessCharacteristic(lightbulbService);
        }
        else {
            this.removeBrightnessCharacteristic(lightbulbService);
        }
        if (supportsColorTemperature) {
            this.addColorTemperatureCharacteristic(lightbulbService);
        }
        else {
            this.removeColorTemperatureCharacteristic(lightbulbService);
        }
        if (supportsColor) {
            this.addColorCharacteristics(lightbulbService);
        }
        else {
            this.removeColorCharacteristics(lightbulbService);
        }
        return lightbulbService;
    }
    addBrightnessCharacteristic(lightbulbService) {
        const brightnessCharacteristic = (0, utils_1.getOrAddCharacteristic)(lightbulbService, this.platform.Characteristic.Brightness);
        brightnessCharacteristic
            .onGet(() => {
            this.getLightState().catch(this.logRejection.bind(this)); // this will eventually trigger update
            const ls = this.tplinkDevice.sysInfo.light_state;
            return ls.brightness ?? ls.dft_on_state?.brightness ?? 0; // immediately returned cached value
        })
            .onSet(async (value) => {
            this.log.info(`Setting Brightness to: ${value}`);
            if (typeof value === 'number') {
                await this.setLightState({ brightness: value });
                return;
            }
            this.log.warn('setValue: Invalid Brightness:', value);
        });
        const brightnessUpdateListener = (lightState) => {
            if (lightState.brightness != null) {
                this.updateValue(lightbulbService, brightnessCharacteristic, lightState.brightness);
            }
        };
        this.tplinkDevice.on('lightstate-update', brightnessUpdateListener);
        this.tplinkDevice.on('lightstate-sysinfo-update', brightnessUpdateListener);
        return brightnessCharacteristic;
    }
    removeBrightnessCharacteristic(service) {
        this.removeCharacteristicIfExists(service, this.platform.Characteristic.Brightness);
    }
    addColorTemperatureCharacteristic(lightbulbService) {
        const range = this.tplinkDevice.colorTemperatureRange;
        if (range == null) {
            this.log.error('Could not retrieve color temperature range');
            return undefined;
        }
        const { min, max } = range;
        const colorTemperatureCharacteristic = (0, utils_1.getOrAddCharacteristic)(lightbulbService, this.platform.Characteristic.ColorTemperature);
        colorTemperatureCharacteristic.setProps({
            minValue: Math.ceil((0, utils_1.kelvinToMired)(max)),
            maxValue: Math.floor((0, utils_1.kelvinToMired)(min)), // K and Mired are reversed
        });
        colorTemperatureCharacteristic
            .onGet(() => {
            this.getLightState().catch(this.logRejection.bind(this)); // this will eventually trigger update
            const ls = this.tplinkDevice.sysInfo.light_state;
            // immediately returned cached value
            if (typeof ls.color_temp === 'number' && ls.color_temp > 0) {
                return Math.round((0, utils_1.kelvinToMired)(ls.color_temp));
            }
            if (typeof ls.dft_on_state?.color_temp === 'number' &&
                ls.dft_on_state.color_temp > 0) {
                return Math.round((0, utils_1.kelvinToMired)(ls.dft_on_state.color_temp));
            }
            if (!('color_temp' in ls)) {
                return Math.floor((0, utils_1.kelvinToMired)(min));
            }
            return Math.floor((0, utils_1.kelvinToMired)(min));
        })
            .onSet(async (value) => {
            this.log.info(`Setting ColorTemperature to: ${value}`);
            if (typeof value === 'number') {
                await this.setLightState({
                    color_temp: Math.round((0, utils_1.miredToKelvin)(value)),
                });
                return;
            }
            this.log.warn('setValue: Invalid ColorTemperature:', value);
        });
        if (colorTemperatureCharacteristic != null) {
            const colorTemperatureUpdateListener = (lightState) => {
                if (lightState.color_temp != null && lightState.color_temp > 0) {
                    this.updateValue(lightbulbService, colorTemperatureCharacteristic, Math.round((0, utils_1.kelvinToMired)(lightState.color_temp)));
                }
            };
            this.tplinkDevice.on('lightstate-update', colorTemperatureUpdateListener);
            this.tplinkDevice.on('lightstate-sysinfo-update', colorTemperatureUpdateListener);
        }
        return colorTemperatureCharacteristic;
    }
    removeColorTemperatureCharacteristic(service) {
        this.removeCharacteristicIfExists(service, this.platform.Characteristic.ColorTemperature);
    }
    addColorCharacteristics(lightbulbService) {
        const hueCharacteristic = (0, utils_1.getOrAddCharacteristic)(lightbulbService, this.platform.Characteristic.Hue);
        hueCharacteristic
            .onGet(() => {
            this.getLightState().catch(this.logRejection.bind(this)); // this will eventually trigger update
            const ls = this.tplinkDevice.sysInfo.light_state;
            return ls.hue ?? ls.dft_on_state?.hue ?? 0; // immediately returned cached value
        })
            .onSet(async (value) => {
            this.log.info(`Setting Hue to: ${value}`);
            if (typeof value === 'number') {
                await this.setLightState({ hue: value, color_temp: 0 });
                return;
            }
            this.log.warn('setValue: Invalid Hue:', value);
        });
        const saturationCharacteristic = (0, utils_1.getOrAddCharacteristic)(lightbulbService, this.platform.Characteristic.Saturation);
        saturationCharacteristic
            .onGet(() => {
            this.getLightState().catch(this.logRejection.bind(this)); // this will eventually trigger update
            const ls = this.tplinkDevice.sysInfo.light_state;
            return ls.saturation ?? ls.dft_on_state?.saturation ?? 0; // immediately returned cached value
        })
            .onSet(async (value) => {
            this.log.info(`Setting Saturation to: ${value}`);
            if (typeof value === 'number') {
                await this.setLightState({ saturation: value, color_temp: 0 });
                return;
            }
            this.log.warn('setValue: Invalid Saturation:', value);
        });
        const colorUpdateListener = (lightState) => {
            if (lightState.color_temp != null && lightState.color_temp > 0) {
                this.updateValue(lightbulbService, hueCharacteristic, 0);
                this.updateValue(lightbulbService, saturationCharacteristic, 0);
            }
            else {
                if (lightState.hue != null) {
                    this.updateValue(lightbulbService, hueCharacteristic, lightState.hue);
                }
                if (lightState.saturation != null) {
                    this.updateValue(lightbulbService, saturationCharacteristic, lightState.saturation);
                }
            }
        };
        this.tplinkDevice.on('lightstate-update', colorUpdateListener);
        this.tplinkDevice.on('lightstate-sysinfo-update', colorUpdateListener);
        return { hueCharacteristic, saturationCharacteristic };
    }
    removeColorCharacteristics(service) {
        [
            this.platform.Characteristic.Hue,
            this.platform.Characteristic.Saturation,
        ].forEach((c) => {
            this.removeCharacteristicIfExists(service, c);
        });
    }
    addEnergyCharacteristics(lightbulbService) {
        const wattsCharacteristic = (0, utils_1.getOrAddCharacteristic)(lightbulbService, this.platform.customCharacteristics.Watts);
        wattsCharacteristic.onGet(() => {
            this.getRealtime().catch(this.logRejection.bind(this)); // this will eventually trigger update
            // immediately returned cached value
            const emeterRealtime = this.tplinkDevice.emeter.realtime;
            if (typeof emeterRealtime.power === 'number') {
                return emeterRealtime.power;
            }
            this.log.warn(`getValue: Invalid Watts:`, emeterRealtime.power);
            return null;
        });
        this.tplinkDevice.on('emeter-realtime-update', (emeterRealtime) => {
            this.updateValue(lightbulbService, wattsCharacteristic, emeterRealtime.power ?? new Error('Could not retrieve watts'));
        });
    }
    removeEnergyCharacteristics(lightbulbService) {
        this.removeCharacteristicIfExists(lightbulbService, this.platform.customCharacteristics.Watts);
    }
    identify() {
        this.log.debug(`identify`);
        (async () => {
            try {
                const origLs = await this.getLightState();
                for (let i = 0; i < 3; i += 1) {
                    /* eslint-disable no-await-in-loop */
                    await this.setLightState({ on_off: 1, brightness: 100 });
                    await (0, utils_1.delay)(500);
                    await this.setLightState({ on_off: 1, brightness: 10 });
                    await (0, utils_1.delay)(500);
                    /* eslint-enable no-await-in-loop */
                }
                this.setLightState(origLs);
            }
            catch (err) {
                this.log.error(`identify error`);
                this.log.error(String(err));
            }
            this.log.debug(`identify complete`);
        })();
    }
}
exports.default = HomeKitDeviceBulb;
//# sourceMappingURL=bulb.js.map