"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = __importDefault(require("chalk"));
const accessory_information_1 = __importDefault(require("../accessory-information"));
const utils_1 = require("../utils");
class HomekitDevice {
    platform;
    config;
    tplinkDevice;
    category;
    log;
    homebridgeAccessory;
    lsc;
    /**
     * Creates an instance of HomeKitDevice.
     */
    constructor(platform, config, homebridgeAccessory, tplinkDevice, category) {
        this.platform = platform;
        this.config = config;
        this.tplinkDevice = tplinkDevice;
        this.category = category;
        this.log = (0, utils_1.prefixLogger)(platform.log, () => `${chalk_1.default.blue(`[${this.name}]`)}`);
        this.lsc = this.platform.lsc.bind(this.platform);
        const categoryName = platform.getCategoryName(category) ?? '';
        if (homebridgeAccessory == null) {
            const uuid = platform.api.hap.uuid.generate(this.id);
            this.log.debug(`Creating new Accessory [${this.id}] [${uuid}] category: ${categoryName}`);
            // eslint-disable-next-line new-cap
            this.homebridgeAccessory = new platform.api.platformAccessory(this.name, uuid, category);
            this.homebridgeAccessory.context.deviceId = this.id;
            this.platform.registerPlatformAccessory(this.homebridgeAccessory);
        }
        else {
            this.homebridgeAccessory = homebridgeAccessory;
            this.log.debug(`Existing Accessory found [${homebridgeAccessory.context.deviceId}] [${homebridgeAccessory.UUID}] category: ${categoryName}`);
            this.homebridgeAccessory.displayName = this.name;
            if (this.homebridgeAccessory.category !== category) {
                this.log.warn(`Correcting Accessory Category from: ${platform.getCategoryName(this.homebridgeAccessory.category)} to: ${categoryName}`);
                this.homebridgeAccessory.category = category;
            }
            this.homebridgeAccessory.context.deviceId = this.id;
            this.platform.api.updatePlatformAccessories([this.homebridgeAccessory]);
        }
        const accInfo = (0, accessory_information_1.default)(platform.api.hap)(this.homebridgeAccessory, this);
        if (accInfo == null) {
            this.log.error('Could not retrieve default AccessoryInformation');
        }
        // Remove Old Services
        this.homebridgeAccessory.services.forEach((service) => {
            if (service instanceof platform.Service.AccessoryInformation)
                return;
            if (service instanceof platform.Service.Lightbulb)
                return;
            if (service instanceof platform.Service.Outlet)
                return;
            if (service instanceof platform.Service.Switch)
                return;
            this.log.warn(`Removing stale Service: ${this.lsc(service)} uuid:[%s] subtype:[%s]`, service.UUID, service.subtype || '');
            this.homebridgeAccessory.removeService(service);
        });
        this.homebridgeAccessory.on("identify" /* PlatformAccessoryEvent.IDENTIFY */, () => {
            this.identify();
        });
    }
    get id() {
        return this.tplinkDevice.id;
    }
    get name() {
        return this.tplinkDevice.alias;
    }
    // eslint-disable-next-line class-methods-use-this
    get manufacturer() {
        return 'TP-Link';
    }
    get model() {
        return this.tplinkDevice.model;
    }
    get serialNumber() {
        return `${this.tplinkDevice.mac} ${this.tplinkDevice.id}`;
    }
    get firmwareRevision() {
        return this.tplinkDevice.softwareVersion;
    }
    get hardwareRevision() {
        return this.tplinkDevice.hardwareVersion;
    }
    updateValue(service, characteristic, value) {
        this.log.debug(`Updating ${this.lsc(service, characteristic)} ${value}`);
        characteristic.updateValue(value);
    }
    addService(serviceConstructor, // WithUUID<Service | typeof Service>,
    name) {
        const serviceName = this.platform.getServiceName(serviceConstructor);
        this.log.debug(`Creating new ${serviceName} Service`);
        return this.homebridgeAccessory.addService(serviceConstructor, name);
    }
    logRejection(reason) {
        this.log.error(JSON.stringify(reason));
    }
    removeServiceIfExists(service) {
        const foundService = this.homebridgeAccessory.getService(service);
        if (foundService != null) {
            this.log.warn(`Removing stale Service: ${this.lsc(service, foundService)} uuid:[%s]`, foundService.UUID);
            this.homebridgeAccessory.removeService(foundService);
        }
    }
    removeCharacteristicIfExists(service, characteristic) {
        // testCharacteristic parameter has an incorrect type
        if (service.testCharacteristic(characteristic)) {
            const characteristicToRemove = service.getCharacteristic(characteristic);
            this.log.warn(`Removing stale Characteristic: ${this.lsc(service, characteristicToRemove)} uuid:[%s]`, characteristicToRemove.UUID);
            service.removeCharacteristic(characteristicToRemove);
        }
    }
}
exports.default = HomekitDevice;
//# sourceMappingURL=index.js.map