"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const _1 = __importDefault(require("."));
const utils_1 = require("../utils");
class HomeKitDevicePlug extends _1.default {
    config;
    tplinkDevice;
    desiredPowerState;
    constructor(platform, config, homebridgeAccessory, tplinkDevice) {
        super(platform, config, homebridgeAccessory, tplinkDevice, (() => {
            if (config.switchModels &&
                config.switchModels.findIndex((m) => tplinkDevice.model.includes(m)) !== -1) {
                return 8 /* Categories.SWITCH */;
            }
            return tplinkDevice.supportsDimmer
                ? 5 /* Categories.LIGHTBULB */
                : 7 /* Categories.OUTLET */;
        })());
        this.config = config;
        this.tplinkDevice = tplinkDevice;
        let primaryService;
        if (tplinkDevice.supportsDimmer) {
            primaryService = this.addLightbulbService();
            this.removeOutletService();
        }
        else {
            primaryService = this.addOutletService();
            this.removeBrightnessCharacteristic(primaryService);
            this.removeLightbulbService();
        }
        if (platform.config.addCustomCharacteristics &&
            tplinkDevice.supportsEmeter) {
            this.addEnergyCharacteristics(primaryService);
        }
        else {
            this.removeEnergyCharacteristics(primaryService);
        }
        this.getSysInfo = (0, utils_1.deferAndCombine)((requestCount) => {
            this.log.debug(`executing deferred getSysInfo count: ${requestCount}`);
            return this.tplinkDevice.getSysInfo();
        }, platform.config.waitTimeUpdate);
        this.setPowerState = (0, utils_1.deferAndCombine)(async (requestCount) => {
            this.log.debug(`executing deferred setPowerState count: ${requestCount}`);
            if (this.desiredPowerState === undefined) {
                this.log.warn('setPowerState called with undefined desiredPowerState');
                return Promise.resolve(true);
            }
            const ret = await this.tplinkDevice.setPowerState(this.desiredPowerState);
            this.desiredPowerState = undefined;
            return ret;
        }, platform.config.waitTimeUpdate, (value) => {
            this.desiredPowerState = value;
        });
        this.getRealtime = (0, utils_1.deferAndCombine)((requestCount) => {
            this.log.debug(`executing deferred getRealtime count: ${requestCount}`);
            return this.tplinkDevice.emeter.getRealtime();
        }, platform.config.waitTimeUpdate);
    }
    /**
     * Aggregates getSysInfo requests
     *
     * @private
     */
    getSysInfo;
    /**
     * Aggregates setPowerState requests
     *
     * @private
     */
    setPowerState;
    /**
     * Aggregates getRealtime requests
     *
     * @private
     */
    getRealtime;
    addOutletService() {
        const { Outlet } = this.platform.Service;
        const { Characteristic } = this.platform;
        const outletService = this.homebridgeAccessory.getService(Outlet) ??
            this.addService(Outlet, this.name);
        this.addOnCharacteristic(outletService);
        if (this.category === 7 /* Categories.OUTLET */) {
            const outletInUseCharacteristic = (0, utils_1.getOrAddCharacteristic)(outletService, Characteristic.OutletInUse);
            outletInUseCharacteristic.onGet(() => {
                this.getSysInfo().catch(this.logRejection.bind(this)); // this will eventually trigger update
                return this.tplinkDevice.inUse; // immediately returned cached value
            });
            this.tplinkDevice.on('in-use-update', (value) => {
                this.updateValue(outletService, outletInUseCharacteristic, value);
            });
        }
        return outletService;
    }
    removeOutletService() {
        this.removeServiceIfExists(this.platform.Service.Outlet);
    }
    addLightbulbService() {
        const { Lightbulb } = this.platform.Service;
        const lightbulbService = this.homebridgeAccessory.getService(Lightbulb) ??
            this.addService(Lightbulb, this.name);
        this.addOnCharacteristic(lightbulbService);
        return lightbulbService;
    }
    removeLightbulbService() {
        this.removeServiceIfExists(this.platform.Service.Lightbulb);
    }
    addOnCharacteristic(service) {
        const onCharacteristic = (0, utils_1.getOrAddCharacteristic)(service, this.platform.Characteristic.On);
        onCharacteristic
            .onGet(() => {
            this.getSysInfo().catch(this.logRejection.bind(this)); // this will eventually trigger update
            return this.tplinkDevice.relayState; // immediately returned cached value
        })
            .onSet(async (value) => {
            this.log.info(`Setting On to: ${value}`);
            if (typeof value === 'boolean') {
                await this.setPowerState(value);
                return;
            }
            this.log.warn('setValue: Invalid On:', value);
            throw new Error(`setValue: Invalid On: ${value}`);
        });
        this.tplinkDevice.on('power-update', (value) => {
            this.updateValue(service, onCharacteristic, value);
        });
        this.addBrightnessCharacteristic(service);
        return service;
    }
    addBrightnessCharacteristic(service) {
        const brightnessCharacteristic = (0, utils_1.getOrAddCharacteristic)(service, this.platform.Characteristic.Brightness);
        brightnessCharacteristic
            .onGet(() => {
            this.getSysInfo().catch(this.logRejection.bind(this)); // this will eventually trigger update
            return this.tplinkDevice.dimmer.brightness; // immediately returned cached value
        })
            .onSet(async (value) => {
            this.log.info(`Setting Brightness to: ${value}`);
            if (typeof value === 'number') {
                await this.tplinkDevice.dimmer.setDimmerTransition({
                    brightness: value,
                    duration: 1,
                });
                return;
            }
            this.log.warn('setValue: Invalid Brightness:', value);
            throw new Error(`setValue: Invalid Brightness: ${value}`);
        });
        this.tplinkDevice.on('brightness-update', (value) => {
            this.updateValue(service, brightnessCharacteristic, value);
        });
        return service;
    }
    removeBrightnessCharacteristic(service) {
        this.removeCharacteristicIfExists(service, this.platform.Characteristic.Brightness);
    }
    addEnergyCharacteristics(service) {
        const { Amperes, KilowattHours, VoltAmperes, Volts, Watts } = this.platform.customCharacteristics;
        const amperesCharacteristic = (0, utils_1.getOrAddCharacteristic)(service, Amperes);
        amperesCharacteristic.onGet(() => {
            this.getRealtime().catch(this.logRejection.bind(this)); // this will eventually trigger update
            return this.tplinkDevice.emeter.realtime.current ?? 0; // immediately returned cached value
        });
        const kilowattCharacteristic = (0, utils_1.getOrAddCharacteristic)(service, KilowattHours);
        kilowattCharacteristic.onGet(() => {
            this.getRealtime().catch(this.logRejection.bind(this)); // this will eventually trigger update
            return this.tplinkDevice.emeter.realtime.total ?? 0; // immediately returned cached value
        });
        const voltAmperesCharacteristic = (0, utils_1.getOrAddCharacteristic)(service, VoltAmperes);
        voltAmperesCharacteristic.onGet(() => {
            this.getRealtime().catch(this.logRejection.bind(this)); // this will eventually trigger update
            const { realtime } = this.tplinkDevice.emeter;
            return (realtime.voltage ?? 0) * (realtime.voltage ?? 0); // immediately returned cached value
        });
        const voltsCharacteristic = (0, utils_1.getOrAddCharacteristic)(service, Volts);
        voltsCharacteristic.onGet(() => {
            this.getRealtime().catch(this.logRejection.bind(this)); // this will eventually trigger update
            return this.tplinkDevice.emeter.realtime.voltage ?? 0; // immediately returned cached value
        });
        const wattsCharacteristic = (0, utils_1.getOrAddCharacteristic)(service, Watts);
        wattsCharacteristic.onGet(() => {
            this.getRealtime().catch(this.logRejection.bind(this)); // this will eventually trigger update
            return this.tplinkDevice.emeter.realtime.power ?? 0; // immediately returned cached value
        });
        this.tplinkDevice.on('emeter-realtime-update', (emeterRealtime) => {
            this.updateValue(service, amperesCharacteristic, emeterRealtime.current ?? null);
            this.updateValue(service, kilowattCharacteristic, emeterRealtime.total ?? null);
            this.updateValue(service, voltAmperesCharacteristic, emeterRealtime.voltage != null && emeterRealtime.current != null
                ? emeterRealtime.voltage * emeterRealtime.current
                : null);
            this.updateValue(service, voltsCharacteristic, emeterRealtime.voltage ?? null);
            this.updateValue(service, wattsCharacteristic, emeterRealtime.power ?? null);
        });
    }
    removeEnergyCharacteristics(service) {
        const { Amperes, KilowattHours, VoltAmperes, Volts, Watts } = this.platform.customCharacteristics;
        [Amperes, KilowattHours, VoltAmperes, Volts, Watts].forEach((characteristic) => {
            this.removeCharacteristicIfExists(service, characteristic);
        });
    }
    identify() {
        this.log.info(`identify`);
        this.tplinkDevice
            .blink(1, 500)
            .then(() => {
            return this.tplinkDevice.blink(2, 500);
        })
            .then(() => {
            this.log.debug(`identify done`);
        })
            .catch((reason) => {
            this.log.error(`identify complete`);
            this.log.error(reason);
        });
    }
}
exports.default = HomeKitDevicePlug;
//# sourceMappingURL=plug.js.map